
-- Fall Winter Demo 2012
-- Kevin Sacro
-- turtlesort.com

-- A really primitive game camera.

-- Given a target in the game, the camera will adjust
-- its own coordinates so that the target is centered
-- on the user's screen.
--
-- The camera will also gracefully glide to targets
-- (unless the target is specifically the player). 

local camera = {}

camera.x      = 0
camera.y      = 0
camera.speed  = 500
camera.target = nil -- Needs to be a table with x and y defined

-- target: a table with x,y coordinates with defined or the player
-- callback: function to execute when camera has centered on the specified
--   x,y coordinates (only works for non-player targets)
function camera.setTarget(target, callback)
	camera.target = target
	camera._targetCallback = callback
	
	if camera.target ~= game.player then
		-- Calculate destination
		camera._targetx = -(target.x) + love.graphics.getWidth()/2
		camera._targety = -(target.y) + love.graphics.getHeight()/2
	end
end


function camera.targetTile(tilex,tiley,callback)
	camera.setTarget(
		{x = tilex * game.tilewidth  + game.tilewidth/2,
		 y = tiley * game.tileheight + game.tileheight/2},
		callback
	)
end

function camera.update(dt)

	if camera.target == game.player then
		-- Adjust camera's x and y so that the player is centered on the screen
		camera.x = -(game.player.x + game.player.width/2) + love.graphics.getWidth()/2
		camera.y = -(game.player.y - game.player.height/2) + love.graphics.getHeight()/2
	else
		-- Move towards the target!
		camera.glide(camera._targetx, camera._targety, dt, camera.speed)
	end

end

-- Expecting dst_x, dst_y as the pixel coordinates of
-- where the camera should center on (with the current map's top
-- left corner as origin); dt is delta time from update(), and
-- speed is a number that will determing how quickly the camera
-- will glide.
function camera.glide(dst_x, dst_y, dt, speed)
	local dx = dst_x - camera.x
	local dy = dst_y - camera.y

	local radians = math.atan2(dy,dx)
	
	dx = speed * math.cos(radians)
	dy = speed * math.sin(radians)

	if camera.x < dst_x then
		camera.x = camera.x + dt * dx
		if camera.x > dst_x then
			camera.x = dst_x;
		end
	end
	if camera.x > dst_x then
		camera.x = camera.x + dt * dx
		if camera.x < dst_x then
			camera.x = dst_x;
		end
	end
	if camera.y < dst_y then
		camera.y = camera.y + dt * dy
		if camera.y > dst_y then
			camera.y = dst_y;
		end
	end
	if camera.y > dst_y then
		camera.y = camera.y + dt * dy
		if camera.y < dst_y then
			camera.y = dst_y;
		end
	end
	
	if camera.x == dst_x and camera.y == dst_y and camera._targetCallback then
		local temp = camera._targetCallback
		camera._targetCallback = nil
		temp()
	end
	
end

return camera